module Joins (start, next, joinComponents, JoinState, isConnected, paths, cliques) where

import Graph
import ListOps
import Control.Parallel.Strategies
import Control.Monad.State
import Control.Monad.Reader
import Control.Monad.Identity

isConnected :: WithGraph Bool
isConnected = do
	graph <- ask
	if vertices graph == [] 
	then return True 
	else return . (~=~ vertices graph) =<< component =<< return . head . vertices =<< return graph

n :: Vertex -> WithGraph [Vertex]
n vertex = do
	graph <- ask
	return $ [v | v <- vertices graph, edge(v,vertex) `elem` edges graph]

ng :: Graph -> Vertex -> [Vertex]
ng graph vertex = [v | v <- vertices graph, edge(v,vertex) `elem` edges graph]

neighbours :: [Vertex] -> WithGraph [Vertex]
neighbours vs = do
	graph <- ask
	return $ [v | v <- vertices graph, not (v `elem` vs), 
		any (`elem` edges graph) [edge(v,w) | w <- vs]]

joinComponents :: Graph -> [Graph]
joinComponents graph = map (graph ~:) . paths . invert $ graph

paths :: Graph -> [[Vertex]]
paths graph = uniquely . parMap rpar (\v -> runReader (component v) graph) . vertices $ graph

component :: Vertex -> WithGraph [Vertex]
component v = pathshelper [v]

pathshelper :: [Vertex] -> WithGraph [Vertex]
pathshelper vs = do
	toAdd <- neighbours vs
	if toAdd == [] then return vs else pathshelper (vs ++ toAdd)

disconnect :: Graph -> [[Vertex]] -> [[Vertex]]
disconnect graph vss
	| vertices graph == [] = vss
	| edges graph == [] = vss ++ [vertices graph]
	| otherwise = vss ++ (disconnect one vss) ++ (disconnect two vss)
	where
		v = head [v | v <- vertices graph, any (`elem` edges graph) [edge (v,w) | w <- vertices graph]]
		one = graph ~: (vertices graph ~-~ [v])
		two = graph ~: (vertices graph ~-~ (runReader (n v) graph))

cliques :: WithGraph [[Vertex]]
cliques = do
	graph <- ask
	let cs = uniquely $ disconnect (invert graph) []
	return $ filter (\ c -> not $ any (c ~<~) (cs ~-~ [c])) cs

type JoinState = ([Graph],[Graph])

type WithJoin a = ReaderT Graph (StateT JoinState Identity) a

scriptC :: [Vertex] -> WithGraph [[Vertex]]
scriptC vs = ask >>= toGraph . (~-~ vs) . vertices >>= return . paths

start :: WithJoin [Graph]
start = do
	invgraph <- ksa
	let joins = withStrategy (parList rpar) . 
		runReader (ask >>= return . vertices >>= mapM (\ v -> return (v : (ng invgraph v))
		>>= scriptC >>=	mapM (neighbours >=> toGraph)) >>= return . foldl (~+~) []) $ invgraph
	put (joins,[])
	graph <- ask
	mapM (toGraph . (vertices graph ~-~) . vertices) joins 
		>>= return . withStrategy (parList $ rparWith rdeepseq)

next :: WithJoin [Graph]
next = do
	(seps,seen) <- get
	invgraph <- ksa
	let notChecked = seps ~-~ seen
	if notChecked == [] then return [] else do
		let newSeps = foldl (~+~) seps . withStrategy (parList rpar) . map (\g ->
			runReader ((return $ vertices g) >>= 
				mapM (mapM (toGraph <=< neighbours) <=< scriptC . (vertices g ~+~) . ng invgraph)
				>>= return . withStrategy (parList rpar) >>= return . foldl (~+~) [])
			invgraph) $ notChecked
		put (newSeps,seps)
		graph <- ask
		mapM (toGraph . (vertices graph ~-~) . vertices) (newSeps ~-~ seps)
			>>= return . withStrategy (parList $ rparWith rdeepseq)
