import System.Random
import System.Environment (getArgs,getProgName)
import System.Exit
import Data.List.Split (chunksOf)

main = do
	args <- getArgs
	prog <- getProgName
	if args == ["-?"] || args == ["-h"] || length args /= 3
	then do
		putStr $ "USAGE: " ++ prog ++ " [-?|-h] number-of-vertices number-of-graphs density\n"
		exitFailure
	else do
		let vertices = read (args !! 0) :: Int
		let graphs 	 = read (args !! 1) :: Int
		let density	 = read (args !! 2) :: Double
		g <- getStdGen
		putStr $ result vertices graphs density g
		return ()

result :: Int -> Int -> Double -> StdGen -> String
result numberOfVertices numberOfGraphs density g
	| density < 0 = result numberOfVertices numberOfGraphs 0 g
	| density > 1 = result numberOfVertices numberOfGraphs 1 g
	| otherwise	  = foldl (\ olds new -> olds ++ new ++ "\n") []
		$ map (\ rs -> (show vs) ++ " " ++ (show $ map snd 
				$ filter ((<= density) . fst) 
				$ zip rs allEdges)) rrs
	where 
		rrs = take numberOfGraphs 
			$ chunksOf ((numberOfVertices * (numberOfVertices - 1)) `div` 2) 
			$ randomRs ((0,1) :: (Double,Double)) g
		vs = take numberOfVertices [1..]
		allEdges = [(v,w) | v <- vs, w <- vs, v < w]
