import Thickness
import ListOps
import Graph (Graph, edge, makeGraph, vertices)
import Control.Exception (catch)
import System.Environment (getArgs,getProgName)
import System.Exit
import Data.Maybe (fromJust)
import Data.List ((\\))

usage = "\nSYNTAX:\tInput one graph per line as\n\t\t[v1,v2,...,vn] [(v11,v12),v(21,v22),...,(vm1,vm2)]"
	++ "\n\twith integers for labels.\n"

note = "\nNOTE:\tDuplicates will be dropped from both lists,\n\tas will edges with any end not contained in the graph\n\tand edges from a vertex to itself.\n"

brief = " -r# -h|u|l|x"

callstyle = brief ++ "\n\nGiven a list of vertices and a list of edges, prints an upper bound on the order of thickness of the corresponding graph or \"n/a\" if the graph is not thick. The upper bound may not be sharp.\n"

readGraph :: String -> Maybe Graph
readGraph input
	| length w /= 2 = Nothing
	| otherwise = Just $ makeGraph (read $ w!!0) (map (edge) . read $ w!!1)
	where w = words input

process :: [String] -> String -> IO (Graph -> String)
process args prog
	| any (`elem` args) ["-h","--help","-?"] = do
		putStr $ "USAGE:\t" ++ prog ++ callstyle ++ usage ++ note 
		exitFailure
	| otherwise = do
		(m,nargs) <- if filter ("-r"~<~) args == [] 
			then return (1,args) 
			else return (read . (\\ "-r") . head . filter ("-r"~<~) $ args,filter (not . ("-r"~<~)) args)
		case nargs of
			["-ux"] -> return $ execThickness True Extra m
			["-u","-x"] -> return $ execThickness True Extra m
			["-x","-u"] -> return $ execThickness True Extra m
			["-ul"] -> return $ execThickness True Long m
			["-u","-l"] -> return $ execThickness True Long m
			["-l","-u"] -> return $ execThickness True Long m
			["-x"] -> return $ execThickness False Extra m
			["-l"] -> return $ execThickness False Long m
			["-u"] -> return $ execThickness True Short m
			[] -> return $ execThickness False Short m
			_ -> do 
				putStr $ "USAGE:\t" ++ prog ++ callstyle ++ usage ++ note
				exitFailure

main = do
	args <- getArgs
	prog <- getProgName
	function <- process args prog
	contents <- getContents
	mapM_ (toTry function) (lines contents) `catch` handler

handler :: IOError -> IO ()
handler _ = putStr usage >> exitFailure

toTry :: (Graph -> String) -> String -> IO ()
toTry function contents = do
	if readGraph contents == Nothing
	then putStr usage >> exitFailure
	else putStr . function . fromJust . readGraph $ contents
